;
; amrom menu
;

.linecont on ; разрыв строки по \
.feature c_comments; разрешение /* комментарий */

.include "nesdefs.inc"


; заголовок
.segment "INESHDR"
    INES_MAPPER = 7 ; 0 = NROM, 2 = UNROM, 7 = AxROM
    INES_MIRROR = 0 ; 0 = horizontal mirroring, 1 = vertical mirroring
    INES_SRAM   = 0 ; 1 = battery backed SRAM at $6000-7FFF
    .byt "NES", $1A ; ID
    .byt 128 ; 16k PRG bank count
    .byt 0 ; 8k CHR bank count
    .byt INES_MIRROR | (INES_SRAM << 1) | ((INES_MAPPER & $f) << 4)
    .byt (INES_MAPPER & %11110000)

; векторы прерываний
.segment "VECTORS"
	.addr nmi_isr, reset, irq_isr

; переменные
.segment "ZEROPAGE"
joy1:           .res 1 ; состояние джойстика
joy1prv:       .res 1 ; предыдущее состояние
tmpvar:         .res 2 ; временная переменная


; код
.segment "CODE"

; прерывание nmi
.proc nmi_isr
    rti
.endproc

; прерывание irq
.proc irq_isr
	rti
.endproc

.proc reset
    sei
    ;cld

    lda #$ff
    tax
    txs ; указатель стека

	inx ; ldx #0
	stx PPU_CTRL		; запрет NMI
	stx PPU_MASK		; отключение спрайтов и фона
    
    ; ждем 2 vblank
    bit PPU_STATUS
:   bit PPU_STATUS
    bpl :-
:   bit PPU_STATUS
    bpl :-

    ; перемещаем все спрайты x=255,y=255
    ;ldx #0
	;lda #$ff
:	sta OAM_RAM, x
	inx
	bne :-
    ; настраиваем спрайт курсора
    sta OAM_RAM+0 ; у
    lda #'@'
    sta OAM_RAM+1 ; номер спрайта
    lda #%00010000 | 2 ; номер палитры 0-3
    sta OAM_RAM+2 ; атрибут
    lda #(8*2)
    sta OAM_RAM+3 ; x


    ; загрузка палитры
    ldx #>$3F00
    stx PPU_ADDR
    ldx #<$3F00
    stx PPU_ADDR

    ;фона
	ldx #0
:	lda palette,x
	sta PPU_DATA
	inx
	cpx #16
	bcc :-

    ; спайтов, та же самая палитра
    ldx #0
:	lda palette,x
	sta PPU_DATA
	inx
	cpx #16
	bcc :-
    

    ; загрузка символов
    lda #0
    ;sty PPU_MASK
    sta PPU_ADDR
    sta PPU_ADDR

    ; 256*3 байт пустоты
    ldx #3 ; для 2bpp.chr
    ;ldx #4 ; для 1bpp.chr
    tay
:   sta PPU_DATA
    iny
    bne :-
    dex
    bne :-


    lda #<chr_tiles
    sta tmpvar
    lda #>chr_tiles
    sta tmpvar+1
    
    ; 768 байт тайлов 2bpp.chr
    ldx #3
    ldy #0
:   lda (tmpvar),y
    sta PPU_DATA
    iny
    bne :-
    inc tmpvar+1
    dex
    bne :-
    
    /*
    ; загрузка 1bpp 32 символа
    ldx #0
@chars_next:
    lda chr_tiles,x
    sta PPU_DATA
    inx
    beq @chars_done
    txa
    and #%111
    eor #%111
    bne @chars_next;  A mod 8 != 0
    ;lda #0
    ldy #8
:   sta PPU_DATA 
    dey
    bne :-
    beq @chars_next
@chars_done:
    */

    ; очистка первой экранной страницы
    ldx #>$2000
    stx PPU_ADDR
    ldx #<$2000
    stx PPU_ADDR
	lda #0
	ldx #32*30/4	; 32*30/4=960/4=240
:	sta PPU_DATA
	sta PPU_DATA
	sta PPU_DATA
	sta PPU_DATA
	dex
	bne :-

    ; палитра для первой строки, PPU_ADDR уже установлен
	ldx #8
	lda #%00000000	; палитра 0
:	sta PPU_DATA
	dex
	bne :-

    ; палитра для второй строки, PPU_ADDR установлен
;	ldx #8
;	lda #%10100101	; палитра 1
;:	sta PPU_DATA
;	dex
;	bne :-

    ; палитра для все остальных строк, PPU_ADDR установлен
	ldx #(64-8)
	lda #%10101010
:	sta PPU_DATA
	dex
	bne :-

    ; загружаем текст меню в 1 страницу
    ldx #>$2000
    stx PPU_ADDR
    ldx #<$2000
    stx PPU_ADDR

    ldy text_menu, x ; количество пробелов
@print_line:
    lda #' '
:   sta PPU_DATA
    dey
    bne :-

    inx
    ldy text_menu, x ; количество символов
:   inx
    lda text_menu, x ; символ
    sta PPU_DATA
    dey
    bne :-

    inx
    ldy text_menu, x ; количество пробелов
    bne @print_line ; пробелов 0 - конец


    ; включение PPU
	lda #BG_0|SPR_0|NT_0|VRAM_RIGHT
	sta PPU_CTRL

	; включение спрайтов и бекграунда
	lda #BG_ON|SPR_ON|HIDE_BG_LHS|HIDE_SPR_LHS
	sta PPU_MASK

    ldy #1 ; выбранный пункт
mainloop:
    
    ; чтение джойстика
    lda #$01
    sta $4016
    sta joy1
    lsr a        ; A=0
    sta $4016
:   lda $4016
    lsr a	    ; bit0 -> Carry
    rol joy1  ; Carry -> bit0; bit 7 -> Carry
    bcc :-


    ; фильтруем только нажатия
    lda joy1
    tax
    eor joy1prv
    and joy1
    stx joy1prv

    ; двигаем курсор
    ;bit:   	 7     6     5     4     3     2     1     0
    ;button:	 A     B  Select Start  Up   Down  Left  Right
    ror a ; 0bit Right
    ror a ; 1bit Left
    ror a ; 2bit Down
    bcc :+
    iny
:   ror a ; 3bit Up
    bcc :+
    dey
  
:   ror a ; 4bit Start
    bcs launch ; запускаем игру
    
    cpy #13
    bne :+
    ldy #1
    
:   cpy #0
    bne :+
    ldy #12
    
:   tya
    clc
    adc #1 ; начинается с 7 строки экрана
    asl a ; a = a*16
    asl a
    asl a
    asl a
    ;clc
    adc #5 ; курсор по центру строки



:   bit PPU_STATUS
    bpl :-

    sta OAM_RAM+0 ; позиция курсора по у
	lda #>OAM_RAM ; вызов DMA
	sta OAM_DMA


    jmp mainloop

launch:
    ; копируем лаунчер в озу
    dey
    
    cpy #11
    bne :+
    jmp tankloader
    
:   ldx #0
:   lda launcher, x
    sta tmpvar, x
    inx
    cmp #$ff ; старший байт от $fffc
    bne :-
    jmp tmpvar

tankloader:
	lda #<chr_tank  ; load the source address into a pointer in zero page
	sta tmpvar
	lda #>chr_tank
	sta tmpvar+1
	
	ldy #0       ; starting index into the first page
	sty PPU_MASK  ; turn off rendering just in case
	sty PPU_ADDR  ; load the destination address into the PPU
	sty PPU_ADDR
	ldx #32      ; number of 256-byte pages to copy
loop:
	lda (tmpvar),y  ; copy one byte
	sta PPU_DATA
	iny
	bne loop  ; repeat until we finish the page
	inc tmpvar+1  ; go to the next page
	dex
	bne loop  ; repeat until we've copied enough pages
	
	ldx #0
:   lda tanklauncher, x
    sta tmpvar, x
    inx
    cmp #$ff ; старший байт от $fffc
    bne :-
    jmp tmpvar
    
tanklauncher:
	lda #$07
	sta	$8000
	lda #0
	
	jmp ($fffc)

launcher:
    ldx latch_high,y
    lda latch_low,y
    
    stx $5000
    sta	$8000
    
    cpy #0
    bne :+
    
    sei
    sta $0022
    jmp ($c4cb)
    
:   jmp ($fffc)

latch_high:
    .byt $00, $00, $01, $02, $03, $04, $05, $06, $06, $07, $07

latch_low:
    .byt $00, $05, $00, $00, $00, $00, $00, $00, $04, $00, $04

palette:
    ; палитра
	.byt $0f, $19, $29, $39; зеленый
	.byt $0f, $18, $28, $38; оранжевый
	.byt $0f, $11, $21, $31; синий
	.byt $0f, $15, $25, $35; розовый

text_menu:
    .include "text_menu.inc"
.endproc


chr_tiles:
	.incbin "alnum.2bpp.chr"
	;.incbin "alnum.1bpp.chr"

; vim:ft=a65:
chr_tank:
	.incbin "tank1990.chr"
